/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2017 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    Blinky.c
 *      Purpose: RTOS2 example program
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>

#include "RTE_Components.h"
#include CMSIS_device_header

/* Atmel Software Package */
#include "board.h"
#include "timer.h"

#include "Board_LED.h"
#include "Board_Buttons.h"

/*----------------------------------------------------------------------------
 *      Main: Initialize and start the RTOS2 Kernel
 *---------------------------------------------------------------------------*/
int main (void) {
  uint32_t led_cnt = LED_GetCount();
  uint32_t led_num = 0;

  /* Board Initialization */
  board_init();

  /* Get Core Clock */
  SystemCoreClockUpdate();

  LED_Initialize();                     /* LED Initialization               */
  Buttons_Initialize();                 /* Buttons Initialization           */

  /* Print message into semihosting channel */
  printf("Blinky Example\n");

  while(1) {

    for (led_num = 0 ; led_num < led_cnt ; led_num++) {

      /* LED on */
      LED_On(led_num);

      /* Wait for 200ms (5Hz) */
  	  msleep(200);
  	  while (Buttons_GetState() != 0);

      /* LED off */
      LED_Off(led_num);
    }

  }
}
