/**************************************************************************//**
 * @file     mmu_SAMA5D2.c
 * @brief    MMU Configuration for ARM Cortex-A5 Device Series
 * @version  V1.00
 * @date     13 Mar 2017
 *
 * @note
 *
 ******************************************************************************/
/*
 * Copyright (c) 2009-2017 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "RTE_Components.h"
#include CMSIS_device_header

//Import symbols from linker
extern uint32_t Image$$VECTORS$$Base;
extern uint32_t Image$$RW_DATA$$Base;
extern uint32_t Image$$ZI_DATA$$Base;
extern uint32_t Image$$TTB$$ZI$$Base;

static uint32_t Sect_Normal;     
static uint32_t Sect_SO;

void MMU_CreateTranslationTable(void)
{
    mmu_region_attributes_Type region;

    // Create faulting entries
    MMU_TTSection (&Image$$TTB$$ZI$$Base, 0, 4096, DESCRIPTOR_FAULT);

    /*
     * Generate descriptors. Refer to core_ca.h to get information about attributes
     *
     */
    // Create descriptors 
    section_normal(Sect_Normal, region);
    section_so(Sect_SO, region);

    /*
     *  Define MMU flat-map regions and attributes
     *
     */
    //Define Image
    /*
    MMU_TTSection (&Image$$TTB$$ZI$$Base, (uint32_t)&Image$$VECTORS$$Base, 1, Sect_Normal);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, (uint32_t)&Image$$RW_DATA$$Base, 1, Sect_Normal);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, (uint32_t)&Image$$ZI_DATA$$Base, 1, Sect_Normal);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, (uint32_t)&Image$$TTB$$ZI$$Base, 1, Sect_Normal);
    */
    //--------------------- PERIPHERALS -------------------

    MMU_TTSection (&Image$$TTB$$ZI$$Base, IROM_ADDR               ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, NFC_RAM_ADDR            ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, IRAM_ADDR               ,   1, Sect_Normal);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, UDPHS_RAM_ADDR          ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, UHPHS_OHCI_ADDR         ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, UHPHS_EHCI_ADDR         ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, AXIMX_ADDR              ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, DAP_ADDR                ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, L2C_310_BASE            ,   2, Sect_SO);

    MMU_TTSection (&Image$$TTB$$ZI$$Base, EBI_CS0_ADDR            , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, DDR_CS_ADDR             ,  64, Sect_Normal);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, DDR_CS_ADDR+0x04000000  , 448, Sect_Normal);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, DDR_AES_CS_ADDR         , 512, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, EBI_CS1_ADDR            , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, EBI_CS2_ADDR            , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, EBI_CS3_ADDR            , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, QSPI_AES0_ADDR          , 128, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, QSPI_AES1_ADDR          , 128, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, SDMMC0_ADDR             , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, SDMMC1_ADDR             , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, NFC_CMD_ADDR            , 256, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, QSPIMEM0_ADDR           , 128, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, QSPIMEM1_ADDR           , 128, Sect_SO);

    MMU_TTSection (&Image$$TTB$$ZI$$Base, 0xF0000000              ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, 0xF8000000              ,   1, Sect_SO);
    MMU_TTSection (&Image$$TTB$$ZI$$Base, 0xFC000000              ,   1, Sect_SO);


    /* Set location of level 1 page table
    ; 31:14 - Translation table base addr (31:14-TTBCR.N, TTBCR.N is 0 out of reset)
    ; 13:7  - 0x0
    ; 6     - IRGN[0] 0x0 (Inner WB WA)
    ; 5     - NOS     0x0 (Non-shared)
    ; 4:3   - RGN     0x1 (Outer WB WA)
    ; 2     - IMP     0x0 (Implementation Defined)
    ; 1     - S       0x0 (Non-shared)
    ; 0     - IRGN[1] 0x1 (Inner WB WA) */
    __set_TTBR0(((uint32_t)&Image$$TTB$$ZI$$Base) | 9);
    __ISB();

    /* Set up domain access control register
    ; We set domain 0 to Client and all other domains to No Access.
    ; All translation table entries specify domain 0 */
    __set_DACR(1);
    __ISB();
}
